package cz.jzitnik.game.entities.items;

import cz.jzitnik.game.annotations.BlockRegistry;
import cz.jzitnik.game.annotations.ItemRegistry;
import cz.jzitnik.game.entities.Block;
import org.reflections.Reflections;

import java.util.HashMap;
import java.util.Set;

public class ItemBlockSupplierNew {
    private static final HashMap<String, Block> registeredBlocks = new HashMap<>();
    private static final HashMap<String, Item> registeredItems = new HashMap<>();

    static {
        registerBlocks();
        registerItems();
    }

    private static void registerBlocks() {
        Reflections reflections = new Reflections("cz.jzitnik.game.entities.items.registry");
        Set<Class<?>> blockClasses = reflections.getTypesAnnotatedWith(BlockRegistry.class);

        for (Class<?> clazz : blockClasses) {
            try {
                Block blockInstance = (Block) clazz.getDeclaredConstructor().newInstance();
                BlockRegistry annotation = clazz.getAnnotation(BlockRegistry.class);
                registeredBlocks.put(annotation.value(), blockInstance);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private static void registerItems() {
        Reflections reflections = new Reflections("cz.jzitnik.game.entities.items.registry");
        Set<Class<?>> itemClasses = reflections.getTypesAnnotatedWith(ItemRegistry.class);

        for (Class<?> clazz : itemClasses) {
            try {
                Item itemInstance = (Item) clazz.getDeclaredConstructor().newInstance();
                ItemRegistry annotation = clazz.getAnnotation(ItemRegistry.class);
                registeredItems.put(annotation.value(), itemInstance);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public static Block getBlock(String id) {
        return registeredBlocks.get(id);
    }

    public static Item getItem(String id) {
        return registeredItems.get(id);
    }
}
